import Axios from "./../Axios";
import AxiosFetcherSearch from "./AxiosFetcherSearch"
import AxiosFetcherOrder from "./AxiosFetcherOrder"
import AxiosFetcherConditions from "./AxiosFetcherConditions";
import AxiosFetcherFilters from "./AxiosFetcherFilters"

/***************
 * AxiosFetcher
 ***************
 * - extragere date dintr-o tabela a BD
 ****************************************/

class AxiosFetcher extends Axios {

    constructor() {
        super()

        /**
         * current_page:integer - Pagina curenta
         */
        this.current_page = 1

        /**
         * per_page:integer - Numar de inregistrari pe pagina 
         */
        this.per_page = null

        /**
         * conditions:AxiosFetcherConditions - conditii pentru filtrare
         */
        this.conditions = new AxiosFetcherConditions()

        /**
         * search:AxiosFetcherSearch - coloane pentru cautare
         */
        this.search = new AxiosFetcherSearch()

        /**
         * order:AxiosFetcherOrder - coloane pentru sortare
         */
        this.order = new AxiosFetcherOrder()

        /**
         * filters:AxiosFetcherFilters - conditii pentru filtrare
         */
        this.filters = new AxiosFetcherFilters()

    }

    set endpoint(value) {
        this._endpoint = value
    }

    get endpoint() {
        if( ! this._endpoint)
        {
            return null
        }
        return ((this._endpoint.substr(0,4) === 'http') ? '' : this.url + '/') + this._endpoint + '?page=' + this.current_page 
    }
    
    get fetcherData() {
        return {
            per_page: this.per_page,
            searchable: this.search.searchable,
            orderable: this.order.currentOrder,
            conditions: this.conditions.conditions,
            filters: this.filters.filterConditions,
        }
    }

    get records() {
        return this.response ? this.response.paginator.data : []
    }

    get from() {
        return this.response ? this.response.paginator.from : null
    }

    get to() {
        return this.response ? this.response.paginator.to : null
    }

    get last_page() {
        return this.response ? this.response.paginator.last_page : null
    }

    get total() {
        return this.response ? this.response.paginator.total : null
    }

    get record_count() {
        return this.response ? this.response.record_count : null
    }

    SetCurrentPage(value) {
        this.current_page = value
        return this
    }

    SetPerPage(value) {
        this.per_page = value
        return this
    }

    SetSearchFields(values) {
        this.search.SetFields(values)
        return this
    }

    AddOrderColumn(column, fields) {
        this.order.AddOrderColumn(column, fields)
        return this
    }

    AddCondition(name, column, type) {
        this.conditions.AddCondition(name, column, type)
        return this
    }

    AddFilter(where, values) {
        this.filters.Add(where, values)
        return this
    }

    /**
     * Methods
     */
    Search(value = null) {
        this.search.value = value
        this.SetCurrentPage(1)
        this.Refresh()
    }

    Order(column, direction) {
        this.order.Order(column, direction)
        this.Refresh()
    }

    Condition(name, current_operator, data) {
        this.conditions.Condition(name, current_operator, data)

        this.SetCurrentPage(1)
        this.Refresh()
    }

    Refresh() {
        this.SetData(this.fetcherData)
        super.Post()
    }
    
}

export default AxiosFetcher